# SPDX-FileCopyrightText: 2025-present woidzero <wwwoidzero@gmail.com>
#
# SPDX-License-Identifier: MIT
"""
## Vanisher
~~~~~~~~
A flexible configuration management library for Python with dot notation support, environment variable overrides, and more.
"""

__version__ = "1.0.4"
